import java.util.HashSet;
import java.util.Set;

/**
 * This class implements a basic generic version of the Observable class
 * presented by Java. It provides all the same elements and is used in basically
 * exactly the same way, except, where necessary, generics are added. The
 * motivation to do this is to make a type-safe version of the Observer pattern
 * that will allow it to be implemented without the need for downcasting.
 * 
 * This class is intended to be subclassed.
 * 
 * @author Matthew Harward
 * @version 11/8/2009
 * @param <U>
 *            The data class being transmitted as observed data.
 * @see Observer<U>
 */
public abstract class Observable<U> {

	// Private field to hold a list of current observers
	private Set<Observer<U>> _observers = new HashSet<Observer<U>>();

	// Private field to keep track of whether the class has changed
	private boolean changed = false;

	/**
	 * Adds a new observer to the set of existing observers.
	 * 
	 * @param obs
	 *            the generic observer to be added.
	 */
	public synchronized void addObserver(Observer<U> obs) {
		if (obs == null) {
			throw new IllegalArgumentException();
		}
		// Note, this will automatically check for duplicates.
		_observers.add(obs);
	}

	/**
	 * A count of the current number of observers.
	 * 
	 * @return the number of current observers.
	 */
	public synchronized int countObservers() {
		return _observers.size();
	}

	/**
	 * Remove all existing observers of this object.
	 */
	public synchronized void deleteObservers() {
		_observers.clear();
	}

	/**
	 * Removes a specified observer from this object.
	 * 
	 * @param obs
	 *            the generic observer to be removed.
	 */
	public synchronized void deleteObserver(Observer<U> obs) {
		if (obs == null) {
			throw new IllegalArgumentException();
		}
		if (_observers.contains(obs)) {
			_observers.remove(obs);
		}
	}

	/**
	 * Clears the change flag to show that no changes have been made.
	 */
	protected synchronized void clearChanged() {
		changed = false;
	}

	/**
	 * @return A boolean value, <code>true</code> if the object has changed,
	 *         else <code>false</code>.
	 */
	public synchronized boolean hasChanged() {
		return changed;
	}

	/**
	 * Declares that their has been an observable change.
	 */
	protected synchronized void setChanged() {
		changed = true;
	}

	/**
	 * Notifies all observers without any passed data. Note, this will mean that
	 * incoming data will be null in the <code>update(..)</code> method.
	 * 
	 * @see <code>notifyObservers(U data)</code> for details.
	 */
	public void notifyObservers() {
		notifyObservers(null);
	}

	/**
	 * If the state of <code>hasChanged</code> is <code>true</code>, all
	 * observers will be notified and subsequently, <code>hasChanged</code> will
	 * return <code>false</code>.
	 * 
	 * @param data
	 *            the generic typed data to be sent as a notification.
	 */
	public void notifyObservers(U data) {
		if (hasChanged()) {
			for (Observer<U> obs : _observers) {
				obs.update(this, data);
			}
			changed = false;
		}

	}
}
